@@ -182,7 +182,7 @@ void powerexp_inplace(arma::mat& res,
 // gneiting 2002 eq. 15 with a,c,beta left unknown
 void gneiting2002_inplace(arma::mat& res, const arma::mat& coords,
                        const arma::uvec& ix, const arma::uvec& iy, 
-                       const double& a, const double& c, const double& beta, const double& sigmasq, bool same){
+                       const double& a, const double& c, const double& beta, const double& sigmasq, const double& nu, bool same){
   // NOT reparametrized here
   //arma::mat res = arma::zeros(ix.n_rows, iy.n_rows);
   arma::uvec timecol = arma::ones<arma::uvec>(1) * 2;
@@ -191,10 +191,26 @@ void gneiting2002_inplace(arma::mat& res, const arma::mat& coords,
       arma::rowvec cri = coords.row(ix(i)).subvec(0, 1); //x.row(i);
       double ti = coords(ix(i), 2);
       for(unsigned int j=i; j<iy.n_rows; j++){
-        double h = arma::norm(cri - coords.submat(iy(j), 0, iy(j), 1)); //y.row(j);
+        double ch = c * arma::norm(cri - coords.submat(iy(j), 0, iy(j), 1)); //y.row(j);
         double u = abs(coords(iy(j), 2) - ti);
         double umod = 1.0 / (a * u + 1.0);
-        res(i, j) = sigmasq * umod * exp(-c * h * pow(umod, beta/2.0) );
+        double chu = ch * pow(umod, beta/2.0);
+        
+        if(ch > 0.0){
+          if(nu == 0.5){
+            res(i, j) = sigmasq * umod * exp(-chu);
+          } else {
+            if(nu == 1.5){
+              res(i, j) = sigmasq * umod * exp(-chu) * (1 + chu);
+            } else {
+              if(nu == 2.5){
+                res(i, j) = sigmasq * umod * (1 + chu + chu*chu / 3.0) * exp(-chu);
+              }
+            }
+          }
+        } else {
+          res(i, j) = sigmasq * umod;
+        }
       }
     }
     res = arma::symmatu(res);
@@ -203,10 +219,26 @@ void gneiting2002_inplace(arma::mat& res, const arma::mat& coords,
       arma::rowvec cri = coords.row(ix(i)).subvec(0, 1); //x.row(i);
       double ti = coords(ix(i), 2);
       for(unsigned int j=0; j<iy.n_rows; j++){
-        double h = arma::norm(cri - coords.submat(iy(j), 0, iy(j), 1)); //y.row(j);
+        double ch = c * arma::norm(cri - coords.submat(iy(j), 0, iy(j), 1)); //y.row(j);
         double u = abs(coords(iy(j), 2) - ti);
         double umod = 1.0 / (a * u + 1.0);
-        res(i, j) = sigmasq * umod * exp(-c * h * pow(umod, beta/2.0) );
+        double chu = ch * pow(umod, beta/2.0);
+        
+        if(ch > 0.0){
+          if(nu == 0.5){
+            res(i, j) = sigmasq * umod * exp(-chu);
+          } else {
+            if(nu == 1.5){
+              res(i, j) = sigmasq * umod * exp(-chu) * (1 + chu);
+            } else {
+              if(nu == 2.5){
+                res(i, j) = sigmasq * umod * (1 + chu + chu*chu / 3.0) * exp(-chu);
+              }
+            }
+          }
+        } else {
+          res(i, j) = sigmasq * umod;
+        }
       }
     }
   }
@@ -299,7 +331,8 @@ arma::mat Correlationf(
     if(matern.using_ps){
       sigmasq = theta(3);
     }
-    gneiting2002_inplace(res, coords, ix, iy, theta(0), theta(1), theta(2), sigmasq, same);
+    double nu = matern.twonu/2.0;
+    gneiting2002_inplace(res, coords, ix, iy, theta(0), theta(1), theta(2), sigmasq, nu, same);
     return res;
   } else {
     // p exposures, p+1 params
